# Copyright © 2021 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script trains an ML pipeline to solve a binary classification task.
"""

from kale.ml import Signature
from kale.sdk import pipeline, step
from kale.common import mlmdutils, artifacts
from sklearn.datasets import make_classification
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split


@step(name="data_loading")
def load(random_state):
    """Create a random dataset for binary classification."""
    rs = int(random_state)
    x, y = make_classification(random_state=rs)
    return x, y


@step(name="data_split")
def split(x, y):
    """Split the data into train and test sets."""
    x, x_test, y, y_test = train_test_split(x, y, test_size=0.1)
    return x, x_test, y, y_test


@step(name="model_training")
def train(x, y, training_iterations):
    """Train a Logistic Regression model."""
    iters = int(training_iterations)
    model = LogisticRegression(max_iter=iters)
    model.fit(x, y)
    return model


@step(name="register_model")
def register_model(model, x, y):
    mlmd = mlmdutils.get_mlmd_instance()

    signature = Signature(
        input_size=[1] + list(x[0].shape),
        output_size=[1] + list(y[0].shape),
        input_dtype=x.dtype,
        output_dtype=y.dtype)

    model_artifact = artifacts.SklearnModel(
        model=model,
        description="A simple Logistic Regression model",
        version="1.0.0",
        author="Kale",
        signature=signature,
        tags={"app": "kale-tutorial"}).submit_artifact()

    mlmd.link_artifact_as_output(model_artifact.id)


@pipeline(name="binary-classification", experiment="kale-tutorial")
def ml_pipeline(rs=42, iters=100):
    """Run the ML pipeline."""
    x, y = load(rs)
    x, x_test, y, y_test = split(x, y)
    model = train(x, y, iters)
    register_model(model, x, y)


if __name__ == "__main__":
    ml_pipeline(rs=42, iters=100)
